package checkers.util;

import javax.lang.model.element.*;
import javax.lang.model.type.*;

import checkers.quals.*;

/**
 * A Utilities class that helps with {@link Element}s.
 *
 */
// TODO: This class needs significant restructuring
@Default("checkers.nonnull.quals.NonNull")
public final class TypesUtils {

    // Cannot be initiated
    private TypesUtils() { }

    /**
     * Gets the fully qualified name for a provided type.  It returns an empty
     * name if type is an anonymous type.
     *
     * @param type the declared type
     * @return the name corresponding to that type
     */
    public static Name getQualifiedName(DeclaredType type) {
        TypeElement element = (TypeElement) type.asElement();
        return element.getQualifiedName();
    }

    /**
     * Checks if the type represents a java.lang.Object declared type
     *
     * @param type  the type
     * @return true iff type represents java.lang.Object
     */
    public static boolean isObject(TypeMirror type) {
        return type.getKind() == TypeKind.DECLARED &&
            getQualifiedName((DeclaredType)type)
            .contentEquals("java.lang.Object");
    }

    /**
     * Checks if the type represents an anonymous type, e.g. as a result of an
     * intersection type
     *
     * @param type  the declared type
     * @return true iff the type represents an anonymous type.
     */
    public static boolean isAnonymousType(TypeMirror type) {
        return ((type.getKind() == TypeKind.DECLARED) &&
                (getQualifiedName((DeclaredType)type).length() == 0));
    }

    public static boolean isBoxedPrimitive(TypeMirror type) {
        if (type.getKind() != TypeKind.DECLARED)
            return false;

        Name qualifiedName = getQualifiedName((DeclaredType)type);

        return (qualifiedName.contentEquals("java.lang.String") ||
                qualifiedName.contentEquals("java.lang.Boolean") ||
                qualifiedName.contentEquals("java.lang.Byte") ||
                qualifiedName.contentEquals("java.lang.Character") ||
                qualifiedName.contentEquals("java.lang.Enum") ||
                qualifiedName.contentEquals("java.lang.Short") ||
                qualifiedName.contentEquals("java.lang.Integer") ||
                qualifiedName.contentEquals("java.lang.Number") ||
                qualifiedName.contentEquals("java.lang.Long") ||
                qualifiedName.contentEquals("java.lang.Double") ||
                qualifiedName.contentEquals("java.lang.Float") ||
                qualifiedName.contentEquals("java.lang.Class"));
    }

    /** @return type represents a Trowable type (e.g. Exception, Error) **/
    public static boolean isThrowable(TypeMirror type) {
        while (type != null && type.getKind() == TypeKind.DECLARED) {
            DeclaredType dt = (DeclaredType) type;
            TypeElement elem = (TypeElement) dt.asElement();
            Name name = elem.getQualifiedName();
            if ("java.lang.Throwable".contentEquals(name))
                return true;
            type = elem.getSuperclass();
        }
        return false;
    }
}
